@@ -1,6 +1,6 @@
 ;;; magit-log.el --- inspect Git history  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2017  The Magit Project Contributors
+;; Copyright (C) 2010-2018  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -32,12 +32,16 @@
 (require 'magit-core)
 (require 'magit-diff)
 
-(declare-function magit-blob-visit 'magit)
-(declare-function magit-insert-head-branch-header 'magit)
-(declare-function magit-insert-upstream-branch-header 'magit)
-(declare-function magit-read-file-from-rev 'magit)
-(declare-function magit-show-commit 'magit)
-(defvar magit-refs-indent-cherry-lines)
+(declare-function magit-blob-visit "magit-files" (blob-or-file line))
+(declare-function magit-insert-head-branch-header "magit-status"
+                  (&optional branch))
+(declare-function magit-insert-upstream-branch-header "magit-status"
+                  (&optional branch pull keyword))
+(declare-function magit-read-file-from-rev "magit-files"
+                  (rev prompt &optional default))
+(declare-function magit-show-commit "magit-diff"
+                  (arg1 &optional arg2 arg3 arg4))
+(defvar magit-refs-focus-column-width)
 (defvar magit-refs-margin)
 (defvar magit-refs-show-commit-count)
 (defvar magit-buffer-margin)
@@ -121,6 +125,20 @@ This is useful if you use really long branch names."
   :group 'magit-log
   :type 'boolean)
 
+(defcustom magit-log-highlight-keywords t
+  "Whether to highlight bracketed keywords in commit summaries."
+  :package-version '(magit . "2.12.0")
+  :group 'magit-log
+  :type 'boolean)
+
+(defcustom magit-log-header-line-function 'magit-log-header-line-sentence
+  "Function used to generate text shown in header line of log buffers."
+  :package-version '(magit . "2.12.0")
+  :group 'magit-log
+  :type '(choice (function-item magit-log-header-line-arguments)
+                 (function-item magit-log-header-line-sentence)
+                 function))
+
 (defface magit-log-graph
   '((((class color) (background light)) :foreground "grey30")
     (((class color) (background  dark)) :foreground "grey80"))
@@ -139,10 +157,15 @@ This is useful if you use really long branch names."
   "Face for the date part of the log output."
   :group 'magit-faces)
 
+(defface magit-header-line-log-select
+  '((t :inherit bold))
+  "Face for the `header-line' in `magit-log-select-mode'."
+  :group 'magit-faces)
+
 ;;;; File Log
 
 (defcustom magit-log-buffer-file-locked t
-  "Whether `magit-log-buffer-file' uses a decicated buffer."
+  "Whether `magit-log-buffer-file' uses a dedicated buffer."
   :package-version '(magit . "2.7.0")
   :group 'magit-commands
   :group 'magit-log
@@ -337,7 +360,8 @@ the upstream isn't ahead of the current branch) show."
                (?S "Show signatures"         "--show-signature")
                (?u "Show diffs"              "--patch")
                (?s "Show diffstats"          "--stat")
-               (?h "Show header"             "++header")
+               (?h "Show header"             "++header" magit-log++header)
+               (?r "Show in reverse order"   "--reverse")
                (?D "Simplify by decoration"  "--simplify-by-decoration")
                (?f "Follow renames when showing single-file log" "--follow"))
     :options  ((?n "Limit number of commits" "-n")
@@ -369,6 +393,7 @@ the upstream isn't ahead of the current branch) show."
                (?S "Show signatures"         "--show-signature")
                (?u "Show diffs"              "--patch")
                (?s "Show diffstats"          "--stat")
+               (?r "Show in reverse order"   "--reverse")
                (?D "Simplify by decoration"  "--simplify-by-decoration")
                (?f "Follow renames when showing single-file log" "--follow"))
     :options  ((?n "Limit number of commits" "-n")
@@ -421,11 +446,21 @@ the upstream isn't ahead of the current branch) show."
       (user-error "Trace is invalid, see man git-log"))))
 
 (defun magit-log-select-order (&rest _ignored)
+  "Set one `--<value>-order' option in Git log.
+This encompasses the options `--author-date-order',
+`--date-order', and `--topo-order'."
   (magit-read-char-case "Order commits by " t
     (?t "[t]opography"     "topo")
     (?a "[a]uthor date"    "author-date")
     (?c "[c]ommitter date" "date")))
 
+;; This is a dummy procedure used to show help in `magit-log-popup'.
+(defun magit-log++header ()
+  "Insert a header after each revision summary in Git log.
+Customize `magit-log-revision-headers-format' to change this
+header."
+  nil)
+
 (defun magit-log-get-buffer-args ()
   (cond ((and magit-use-sticky-arguments
               (derived-mode-p 'magit-log-mode))
@@ -639,6 +674,7 @@ active, restrict the log to the lines that the region touches."
   (interactive
    (cons current-prefix-arg
          (and (region-active-p)
+              (magit-file-relative-name)
               (save-restriction
                 (widen)
                 (list (line-number-at-pos (region-beginning))
@@ -659,7 +695,9 @@ active, restrict the log to the lines that the region touches."
              (let ((args (car (magit-log-arguments))))
                (when (and follow (not (member "--follow" args)))
                  (push "--follow" args))
-               (when (and beg end)
+               (when (and (file-regular-p
+                           (expand-file-name file (magit-toplevel)))
+                          beg end)
                  (setq args (cons (format "-L%s,%s:%s" beg end file)
                                   (cl-delete "-L" args :test
                                              'string-prefix-p)))
@@ -670,6 +708,39 @@ active, restrict the log to the lines that the region touches."
     (user-error "Buffer isn't visiting a file"))
   (magit-log-goto-same-commit))
 
+;;;###autoload
+(defun magit-log-trace-definition (file fn rev)
+  "Show log for the definition at point."
+  (interactive (list (or (magit-file-relative-name)
+                         (user-error "Buffer isn't visiting a file"))
+                     (add-log-current-defun)
+                     (or magit-buffer-refname
+                         (magit-get-current-branch)
+                         "HEAD")))
+  (require 'magit)
+  (magit-mode-setup-internal
+   #'magit-log-mode
+   (list (list rev)
+         (cons (format "-L:%s:%s" fn file)
+               (cl-delete "-L" (car (magit-log-arguments))
+                          :test 'string-prefix-p))
+         nil)
+   magit-log-buffer-file-locked)
+  (magit-log-goto-same-commit))
+
+(defun magit-diff-trace-definition ()
+  "Show log for the definition at point in a diff."
+  (interactive)
+  (let (buf pos)
+    (save-window-excursion
+      (call-interactively #'magit-diff-visit-file)
+      (setq buf (current-buffer))
+      (setq pos (point)))
+    (save-excursion
+      (with-current-buffer buf
+        (goto-char pos)
+        (call-interactively #'magit-log-trace-definition)))))
+
 ;;;###autoload
 (defun magit-reflog-current ()
   "Display the reflog of the current branch."
@@ -749,9 +820,9 @@ is displayed in the current frame."
   (interactive "p")
   (when (derived-mode-p 'magit-log-mode)
     (magit-section-when commit
-      (let ((parent-rev (format "%s^%s" (magit-section-value it) (or n 1))))
+      (let ((parent-rev (format "%s^%s" (oref it value) (or n 1))))
         (-if-let (parent-hash (magit-rev-parse "--short" parent-rev))
-            (-if-let (section (--first (equal (magit-section-value it)
+            (-if-let (section (--first (equal (oref it value)
                                               parent-hash)
                                        (magit-section-siblings it 'next)))
                 (magit-section-goto section)
@@ -806,16 +877,11 @@ Type \\[magit-reset] to reset `HEAD' to the commit at point.
   "Arguments which disable the graph speedup hack.")
 
 (defun magit-log-refresh-buffer (revs args files)
-  (setq header-line-format
-        (propertize
-         (concat " Commits in " (mapconcat 'identity revs  " ")
-                 (and files (concat " touching "
-                                    (mapconcat 'identity files " ")))
-                 (--some (and (string-prefix-p "-L" it)
-                              (concat " " it))
-                         args))
-         'face 'magit-header-line))
-  (unless (= (length files) 1)
+  (magit-set-header-line-format
+   (funcall magit-log-header-line-function revs args files))
+  (if (= (length files) 1)
+      (unless (magit-file-tracked-p (car files))
+        (setq args (cons "--full-history" args)))
     (setq args (remove "--follow" args)))
   (when (--any-p (string-match-p
                   (concat "^" (regexp-opt magit-log-remove-graph-args)) it)
@@ -842,6 +908,27 @@ Type \\[magit-reset] to reset `HEAD' to the commit at point.
   (magit-insert-section (logbuf)
     (magit-insert-log revs args files)))
 
+(defun magit-log-header-line-arguments (revs args files)
+  "Return string describing some of the used arguments."
+  (mapconcat (lambda (arg)
+               (if (string-match-p " " arg)
+                   (prin1 arg)
+                 arg))
+             `("git" "log" ,@args ,@revs "--" ,@files)
+             " "))
+
+(defun magit-log-header-line-sentence (revs args files)
+  "Return string containing all arguments."
+  (concat "Commits in "
+          (mapconcat #'identity revs " ")
+          (and (member "--reverse" args)
+               " in reverse")
+          (and files (concat " touching "
+                             (mapconcat 'identity files " ")))
+          (--some (and (string-prefix-p "-L" it)
+                       (concat " " it))
+                  args)))
+
 (defun magit-insert-log (revs &optional args files)
   "Insert a log section.
 Do not add this to a hook variable."
@@ -863,9 +950,11 @@ Do not add this to a hook variable."
         (--when-let (--first (string-match "^\\+\\+order=\\(.+\\)$" it) args)
           (setq args (cons (format "--%s-order" (match-string 1 it))
                            (remove it args))))
-        (if (member "--decorate" args)
-            (cons "--decorate=full" (remove "--decorate" args))
-          args))
+        (when (member "--decorate" args)
+          (setq args (cons "--decorate=full" (remove "--decorate" args))))
+        (when (member "--reverse" args)
+          (setq args (remove "--graph" args)))
+        args)
       "--use-mailmap" "--no-prefix" revs "--" files)))
 
 (defvar magit-commit-section-map
@@ -938,6 +1027,8 @@ Do not add this to a hook variable."
 
 (defvar magit-log-count nil)
 
+(defvar magit-log-format-message-function 'magit-log-propertize-keywords)
+
 (defun magit-log-wash-log (style args)
   (setq args (-flatten args))
   (when (and (member "--graph" args)
@@ -949,9 +1040,9 @@ Do not add this to a hook variable."
       (ansi-color-apply-on-region (point-min) (point-max))))
   (when (eq style 'cherry)
     (reverse-region (point-min) (point-max)))
-  (let ((magit-log-count 0)
-        (abbrev (magit-abbrev-length)))
-    (magit-wash-sequence (apply-partially 'magit-log-wash-rev style abbrev))
+  (let ((magit-log-count 0))
+    (magit-wash-sequence (apply-partially 'magit-log-wash-rev style
+                                          (magit-abbrev-length)))
     (if (derived-mode-p 'magit-log-mode)
         (when (eq magit-log-count (magit-log-get-commit-limit))
           (magit-insert-section (longer)
@@ -979,6 +1070,9 @@ Do not add this to a hook variable."
                 (`bisect-log magit-log-bisect-log-re)))
   (magit-bind-match-strings
       (hash msg refs graph author date gpg cherry _ refsub side) nil
+    (setq msg (substring-no-properties msg))
+    (when refs
+      (setq refs (substring-no-properties refs)))
     (let ((align (not (member "--stat" (cadr magit-refresh-args))))
           (non-graph-re (if (eq style 'bisect-vis)
                             magit-log-bisect-vis-re
@@ -991,13 +1085,13 @@ Do not add this to a hook variable."
         (cl-return-from magit-log-wash-rev t))
       (magit-insert-section section (commit hash)
         (pcase style
-          (`stash      (setf (magit-section-type section) 'stash))
-          (`module     (setf (magit-section-type section) 'module-commit))
+          (`stash      (oset section type 'stash))
+          (`module     (oset section type 'module-commit))
           (`bisect-log (setq hash (magit-rev-parse "--short" hash))))
         (when cherry
           (when (and (derived-mode-p 'magit-refs-mode)
                      magit-refs-show-commit-count)
-            (insert (make-string magit-refs-indent-cherry-lines ?\s)))
+            (insert (make-string (1- magit-refs-focus-column-width) ?\s)))
           (insert (propertize cherry 'face (if (string= cherry "-")
                                                'magit-cherry-equivalent
                                              'magit-cherry-unmatched)))
@@ -1031,13 +1125,7 @@ Do not add this to a hook variable."
                                     (?Y 'magit-signature-expired-key)
                                     (?R 'magit-signature-revoked)
                                     (?E 'magit-signature-error)))))
-          (let ((start 0))
-            (while (string-match "\\[[^[]*\\]" msg start)
-              (setq start (match-end 0))
-              (put-text-property (match-beginning 0)
-                                 (match-end 0)
-                                 'face 'magit-keyword msg)))
-          (insert msg))
+          (insert (funcall magit-log-format-message-function hash msg)))
         (when (and refs magit-log-show-refname-after-summary)
           (insert ?\s)
           (insert (magit-format-ref-labels refs)))
@@ -1076,7 +1164,7 @@ Do not add this to a hook variable."
               (let ((limit (save-excursion
                              (and (re-search-forward non-graph-re nil t)
                                   (match-beginning 0)))))
-                (unless (magit-section-content magit-insert-section--current)
+                (unless (oref magit-insert-section--current content)
                   (magit-insert-heading))
                 (delete-char (if (looking-at "\n") 1 4))
                 (magit-diff-wash-diffs (list "--stat") limit))
@@ -1101,6 +1189,16 @@ Do not add this to a hook variable."
                 (insert graph ?\n))))))))
   t)
 
+(defun magit-log-propertize-keywords (_rev msg)
+  (let ((start 0))
+    (while (string-match "\\[[^[]*\\]" msg start)
+      (setq start (match-end 0))
+      (when magit-log-highlight-keywords
+        (put-text-property (match-beginning 0)
+                           (match-end 0)
+                           'face 'magit-keyword msg))))
+  msg)
+
 (defun magit-log-maybe-show-more-commits (section)
   "When point is at the end of a log buffer, insert more commits.
 
@@ -1111,7 +1209,7 @@ having to press \"+\".
 
 This function is called by `magit-section-movement-hook' and
 exists mostly for backward compatibility reasons."
-  (when (and (eq (magit-section-type section) 'longer)
+  (when (and (eq (oref section type) 'longer)
              magit-log-auto-more)
     (magit-log-double-commit-limit)
     (forward-line -1)
@@ -1172,13 +1270,12 @@ If there is no blob buffer in the same frame, then do nothing."
 (defun magit-log-goto-same-commit ()
   (-when-let* ((prev magit-previous-section)
                (rev  (cond ((magit-section-match 'commit prev)
-                            (magit-section-value prev))
+                            (oref prev value))
                            ((magit-section-match 'branch prev)
-                            (magit-rev-format
-                             "%h" (magit-section-value prev)))))
-               (same (--first (equal (magit-section-value it) rev)
-                              (magit-section-children magit-root-section))))
-    (goto-char (magit-section-start same))))
+                            (magit-rev-format "%h" (oref prev value)))))
+               (same (--first (equal (oref it value) rev)
+                              (oref magit-root-section children))))
+    (goto-char (oref same start))))
 
 ;;; Log Margin
 
@@ -1261,28 +1358,36 @@ Type \\[magit-log-select-quit] to abort without selecting a commit."
 (defvar-local magit-log-select-pick-function nil)
 (defvar-local magit-log-select-quit-function nil)
 
-(defun magit-log-select (pick &optional msg quit branch)
+(defun magit-log-select (pick &optional msg quit branch args)
   (declare (indent defun))
   (magit-mode-setup #'magit-log-select-mode
                     (or branch (magit-get-current-branch) "HEAD")
-                    magit-log-select-arguments)
+                    (append args magit-log-select-arguments))
   (magit-log-goto-same-commit)
   (setq magit-log-select-pick-function pick)
   (setq magit-log-select-quit-function quit)
   (when magit-log-select-show-usage
-    (setq msg (substitute-command-keys
-               (format-spec
-                (if msg
-                    (if (string-suffix-p "," msg)
-                        (concat msg " or %q to abort")
-                      msg)
-                  "Type %p to select commit at point, or %q to abort")
-                '((?p . "\\[magit-log-select-pick]")
-                  (?q . "\\[magit-log-select-quit]")))))
+    (let ((pick (propertize (substitute-command-keys
+                             "\\[magit-log-select-pick]")
+                            'face
+                            'magit-header-line-key))
+          (quit (propertize (substitute-command-keys
+                             "\\[magit-log-select-quit]")
+                            'face
+                            'magit-header-line-key)))
+      (setq msg (format-spec
+                 (if msg
+                     (if (string-suffix-p "," msg)
+                         (concat msg " or %q to abort")
+                       msg)
+                   "Type %p to select commit at point, or %q to abort")
+                 `((?p . ,pick)
+                   (?q . ,quit)))))
+    (add-face-text-property 0 (length msg) 'magit-header-line-log-select t msg)
     (when (memq magit-log-select-show-usage '(both header-line))
-      (setq header-line-format (propertize (concat " " msg) 'face 'bold)))
+      (magit-set-header-line-format msg))
     (when (memq magit-log-select-show-usage '(both echo-area))
-      (message "%s" msg))))
+      (message "%s" (substring-no-properties msg)))))
 
 (defun magit-log-select-pick ()
   "Select the commit at point and act on it.
@@ -1291,13 +1396,13 @@ commit as argument."
   (interactive)
   (let ((fun magit-log-select-pick-function)
         (rev (magit-commit-at-point)))
-    (quit-restore-window nil 'kill)
+    (magit-mode-bury-buffer 'kill)
     (funcall fun rev)))
 
 (defun magit-log-select-quit ()
   "Abort selecting a commit, don't act on any commit."
   (interactive)
-  (quit-restore-window nil 'kill)
+  (magit-mode-bury-buffer 'kill)
   (when magit-log-select-quit-function
     (funcall magit-log-select-quit-function)))
 
@@ -1387,8 +1492,7 @@ Type \\[magit-reset] to reset `HEAD' to the commit at point.
               'magit-bookmark--reflog-make-record))
 
 (defun magit-reflog-refresh-buffer (ref args)
-  (setq header-line-format
-        (propertize (concat " Reflog for " ref) 'face 'magit-header-line))
+  (magit-set-header-line-format (concat "Reflog for " ref))
   (magit-insert-section (reflogbuf)
     (magit-git-wash (apply-partially 'magit-log-wash-log 'reflog)
       "reflog" "show" "--format=%h%x00%aN%x00%gd%x00%gs" "--date=raw"
@@ -1476,32 +1580,43 @@ Type \\[magit-reset] to reset `HEAD' to the commit at point.
 (magit-define-section-jumper magit-jump-to-unpushed-to-upstream
   "Unpushed to @{upstream}" unpushed "@{upstream}..")
 
-(defun magit-insert-unpushed-to-upstream ()
+(defun magit-insert-unpushed-to-upstream-or-recent ()
+  "Insert section showing unpushed or other recent commits.
+If an upstream is configured for the current branch and it is
+behind of the current branch, then show the commits that have
+not yet been pushed into the upstream branch.  If no upstream is
+configured or if the upstream is not behind of the current branch,
+then show the last `magit-log-section-commit-count' commits."
+  (let ((upstream (magit-rev-parse "@{upstream}")))
+    (if (or (not upstream)
+            (magit-rev-ancestor-p "HEAD" upstream))
+        (magit-insert-recent-commits
+         (--if-let (and magit-insert-section--oldroot
+                        (magit-get-section
+                         '((unpushed . "@{upstream}..")
+                           (status))
+                         magit-insert-section--oldroot))
+             (oref it hidden)
+           nil))
+      (magit-insert-unpushed-to-upstream
+       (--if-let (and magit-insert-section--oldroot
+                      (magit-get-section
+                       `((recent . ,(format "HEAD~%s..HEAD"
+                                            magit-log-section-commit-count))
+                         (status))
+                       magit-insert-section--oldroot))
+           (oref it hidden)
+         t)))))
+
+(defun magit-insert-unpushed-to-upstream (&optional collapse)
   "Insert commits that haven't been pushed to the upstream yet."
   (when (magit-git-success "rev-parse" "@{upstream}")
-    (magit-insert-section (unpushed "@{upstream}..")
+    (magit-insert-section (unpushed "@{upstream}.." collapse)
       (magit-insert-heading
         (format (propertize "Unmerged into %s:" 'face 'magit-section-heading)
                 (magit-get-upstream-branch)))
       (magit-insert-log "@{upstream}.." magit-log-section-arguments))))
 
-(magit-define-section-jumper magit-jump-to-unpushed-to-pushremote
-  "Unpushed to <push-remote>" unpushed
-  (concat (magit-get-push-branch) ".."))
-
-(defun magit-insert-unpushed-to-pushremote ()
-  "Insert commits that haven't been pushed to the push-remote yet."
-  (--when-let (magit-get-push-branch)
-    (unless (and (equal (magit-rev-name it)
-                        (magit-rev-name "@{upstream}"))
-                 (memq 'magit-insert-unpushed-to-upstream
-                       magit-status-sections-hook))
-      (magit-insert-section (unpushed (concat it ".."))
-        (magit-insert-heading
-          (format (propertize "Unpushed to %s:" 'face 'magit-section-heading)
-                  (propertize it 'face 'magit-branch-remote)))
-        (magit-insert-log (concat it "..") magit-log-section-arguments)))))
-
 (defun magit-insert-recent-commits (&optional collapse)
   "Insert section showing recent commits.
 Show the last `magit-log-section-commit-count' commits."
@@ -1511,21 +1626,28 @@ Show the last `magit-log-section-commit-count' commits."
     (magit-insert-section (recent range collapse)
       (magit-insert-heading "Recent commits")
       (magit-insert-log range
-                        (cons (format "-%d" magit-log-section-commit-count)
-                              magit-log-section-arguments)))))
+                        (cons (format "-n%d" magit-log-section-commit-count)
+                              (--remove (string-prefix-p "-n" it)
+                                        magit-log-section-arguments))))))
 
-(defun magit-insert-unpulled-from-upstream-or-recent ()
-  "Insert section showing unpulled or recent commits.
-If an upstream is configured for the current branch and it is
-ahead of the current branch, then show the commits that have
-not yet been pulled into the current branch.  If no upstream is
-configured or if the upstream is not ahead of the current branch,
-then show the last `magit-log-section-commit-count' commits."
-  (let ((upstream (magit-rev-parse "@{upstream}")))
-    (if (or (not upstream)
-            (equal upstream (magit-rev-parse "HEAD")))
-        (magit-insert-recent-commits t)
-      (magit-insert-unpulled-from-upstream))))
+(magit-define-section-jumper magit-jump-to-unpushed-to-pushremote
+  "Unpushed to <push-remote>" unpushed
+  (concat (magit-get-push-branch) ".."))
+
+(defun magit-insert-unpushed-to-pushremote ()
+  "Insert commits that haven't been pushed to the push-remote yet."
+  (--when-let (magit-get-push-branch)
+    (unless (and (equal (magit-rev-name it)
+                        (magit-rev-name "@{upstream}"))
+                 (or (memq 'magit-insert-unpushed-to-upstream
+                           magit-status-sections-hook)
+                     (memq 'magit-insert-unpushed-to-upstream-or-recent
+                           magit-status-sections-hook)))
+      (magit-insert-section (unpushed (concat it ".."))
+        (magit-insert-heading
+          (format (propertize "Unpushed to %s:" 'face 'magit-section-heading)
+                  (propertize it 'face 'magit-branch-remote)))
+        (magit-insert-log (concat it "..") magit-log-section-arguments)))))
 
 ;;;; Auxiliary Log Sections
 
