@@ -1,6 +1,6 @@
 ;;; magit-worktree.el --- worktree support  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2017  The Magit Project Contributors
+;; Copyright (C) 2010-2018  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -34,16 +34,57 @@
 
 ;;; Commands
 
+;;;###autoload (autoload 'magit-worktree-popup "magit-worktree" nil t)
+(magit-define-popup magit-worktree-popup
+  "Popup console for worktree commands."
+  :man-page "git-worktree"
+  :actions  '((?b "Create new worktree"            magit-worktree-checkout)
+              (?c "Create new branch and worktree" magit-worktree-branch)
+              (?p "Create new worktree from pull-request"
+                  magit-worktree-checkout-pull-request)
+              (?k "Delete worktree"                magit-worktree-delete)
+              (?g "Show status for worktree"       magit-worktree-status))
+  :max-action-columns 1)
+
 ;;;###autoload
 (defun magit-worktree-checkout (path branch)
   "Checkout BRANCH in a new worktree at PATH."
   (interactive
-   (let ((branch (magit-read-local-branch "Checkout")))
+   (let ((branch (magit-read-local-branch-or-commit "Checkout")))
      (list (read-directory-name (format "Checkout %s in new worktree: " branch))
            branch)))
   (magit-run-git "worktree" "add" (expand-file-name path) branch)
   (magit-diff-visit-directory path))
 
+(defun magit-worktree-checkout-pull-request (path pr)
+  "Create, configure and checkout a new worktree from a pull-request.
+This is like `magit-checkout-pull-request', except that it
+also creates a new worktree. Please see the manual for more
+information."
+  (interactive
+   (let ((pr (magit-read-pull-request "Checkout pull request")))
+     (let-alist pr
+       (let ((path (let ((branch (magit--pullreq-branch pr t)))
+                     (read-directory-name
+                      (format "Checkout #%s as `%s' in new worktree: "
+                              .number branch)
+                      (file-name-directory
+                       (directory-file-name default-directory))
+                      nil nil
+                      (if (string-match-p "\\`pr-[0-9]+\\'" branch)
+                          (number-to-string .number)
+                        (format "%s-%s" .number .head.ref))))))
+         (when (equal path "")
+           (user-error "The empty string isn't a valid path"))
+         (list path pr)))))
+  (when (and (file-exists-p path)
+             (not (and (file-directory-p path)
+                       (= (length (directory-files "/tmp/testing/")) 2))))
+    (user-error "%s already exists and isn't empty" path))
+  (magit-worktree-checkout path
+                           (let ((inhibit-magit-refresh t))
+                             (magit-branch-pull-request pr))))
+
 ;;;###autoload
 (defun magit-worktree-branch (path branch start-point &optional force)
   "Create a new BRANCH and check it out in a new worktree at PATH."
@@ -66,15 +107,18 @@ The primary worktree cannot be deleted."
   (if (file-directory-p (expand-file-name ".git" worktree))
       (user-error "Deleting %s would delete the shared .git directory" worktree)
     (let ((primary (file-name-as-directory (caar (magit-list-worktrees)))))
-      (when (if magit-delete-by-moving-to-trash
-                (magit-confirm-files 'trash (list "worktree"))
-              (magit-confirm-files 'delete (list "worktree")))
+      (magit-confirm-files (if magit-delete-by-moving-to-trash 'trash 'delete)
+                           (list "worktree"))
+      (when (file-exists-p worktree)
         (let ((delete-by-moving-to-trash magit-delete-by-moving-to-trash))
-          (delete-directory worktree t magit-delete-by-moving-to-trash))
-        (if (file-exists-p default-directory)
-            (magit-run-git "worktree" "prune")
-          (let ((default-directory primary))
-            (magit-run-git "worktree" "prune")))))))
+          (delete-directory worktree t magit-delete-by-moving-to-trash)))
+      (if (file-exists-p default-directory)
+          (magit-run-git "worktree" "prune")
+        (let ((default-directory primary))
+          (magit-run-git "worktree" "prune"))
+        (when (derived-mode-p 'magit-status-mode)
+          (kill-buffer)
+          (magit-status-internal primary))))))
 
 (defun magit-worktree-status (worktree)
   "Show the status for the worktree at point.
